"use strict";
const hrtime_now_1 = require("hrtime-now");
const typescript_ast_util_1 = require("typescript-ast-util");
const typescript_plugin_util_1 = require("typescript-plugin-util");
const refactors_1 = require("./refactors");
const log = (msg) => info.project.projectService.logger.info(`${refactors_1.PLUGIN_NAME} ${msg}`);
function getCompletionsAtPosition(fileName, position, options) {
    const t0 = hrtime_now_1.now();
    const prior = info.languageService.getCompletionsAtPosition(fileName, position, options);
    getAllRefactors().forEach(refactor => {
        prior.entries = prior.entries.concat(refactor.getCompletionsAtPosition(fileName, position, options));
    });
    log(`getCompletionsAtPosition took ${hrtime_now_1.timeFrom(t0)}`);
    return prior;
}
let opts;
function getApplicableRefactors(fileName, positionOrRange, userPreferences) {
    const t0 = hrtime_now_1.now();
    let refactors = info.languageService.getApplicableRefactors(fileName, positionOrRange, userPreferences);
    const program = info.languageService.getProgram();
    const sourceFile = program.getSourceFile(fileName);
    if (!sourceFile) {
        log(`getApplicableRefactors return empty because !sourceFile`);
        return;
    }
    const diagnostics = [];
    const start = typescript_ast_util_1.positionOrRangeToNumber(positionOrRange);
    const range = typescript_ast_util_1.positionOrRangeToRange(start + 1);
    const containingTarget = typescript_ast_util_1.findChildContainingRange(sourceFile, range) || sourceFile;
    const containingTargetLight = typescript_ast_util_1.findChildContainingRangeLight(sourceFile, range) || sourceFile;
    const containedTarget = typescript_ast_util_1.findChildContainedRange(sourceFile, range) || sourceFile;
    opts = { diagnostics, containingTarget, containingTargetLight, containedTarget, log, program, sourceFile, positionOrRange };
    getAllRefactors()
        .filter(refactor => {
        try {
            return refactor.predicate(opts);
        }
        catch (ex) {
            log('getCodeFix exception in plugin predicate ' + refactor.name + ex + '\n' + ex.stack);
        }
    })
        .filter(r => !!r)
        .forEach(refactor => {
        refactors = refactors.concat(refactor.getApplicableRefactors(info, `${refactors_1.PLUGIN_NAME}`, `${refactors_1.PLUGIN_NAME}-${refactor.name}`, fileName, positionOrRange, userPreferences).refactors);
    });
    log(`getApplicableRefactors took ${hrtime_now_1.timeFrom(t0)}`);
    return refactors;
}
function getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName, userPreferences) {
    const t0 = hrtime_now_1.now();
    const refactors = info.languageService.getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName, userPreferences);
    if (actionName.startsWith(refactors_1.PLUGIN_NAME)) {
        const name = actionName.substring(refactors_1.PLUGIN_NAME.length + 1, actionName.length);
        const refactor = getAllRefactors().find(r => r.name === name);
        if (!refactor) {
            log(`getEditsForRefactor abort because refactor named ${name} not found`);
            return refactors;
        }
        try {
            log(`getEditsForRefactor refactor found ${refactor.name}`);
            opts.simpleProject = typescript_plugin_util_1.createSimpleASTProject(info.project);
            const sourceFile = opts.simpleProject.getSourceFileOrThrow(fileName);
            if (sourceFile.getFullText() !== opts.sourceFile.getFullText()) {
                sourceFile.replaceWithText(opts.sourceFile.getFullText()); // update tsa sourcefile with buffer contents
            }
            opts.simpleNode = sourceFile.getDescendantAtPos(typescript_ast_util_1.positionOrRangeToNumber(positionOrRange)) || sourceFile;
            refactor.apply(opts);
        }
        catch (error) {
            log(`getEditsForRefactor error ${error + ' - ' + error.stack}`);
        }
        log(`getEditsForRefactor ${actionName} took  ${hrtime_now_1.timeFrom(t0)}`);
    }
    return refactors;
}
let info;
const pluginDefinition = {
    getApplicableRefactors, getEditsForRefactor, getCompletionsAtPosition
};
function getAllRefactors() {
    return refactors_1.getRefactors(getRefactorsConfig());
}
function getRefactorsConfig() {
    return { log, info };
}
module.exports = typescript_plugin_util_1.getPluginCreate(pluginDefinition, (modules, anInfo) => {
    info = anInfo;
    log(`created`);
});
//# sourceMappingURL=index.js.map