"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const typescript_plugins_text_based_user_interaction_1 = require("typescript-plugins-text-based-user-interaction");
const refactors_1 = require("../../refactors");
const reorderParams_1 = require("./reorderParams");
const input_ui_ipc_provider_1 = require("input-ui-ipc-provider");
/**
 * # Description
 *
 *  Allows users to change a signature parameter order. see README
 *
 * # TODO
 *  * refactor a implementation method wont change its interface signature - super
 *  * constructors not supported
 */
class ReorderParamsCodeFixImpl {
    constructor(options) {
        this.options = options;
        this.name = refactors_1.PLUGIN_NAME + '-reorderParams';
        this.config = {
            // print a comment with help on the autocomplete suggestion
            help: false
        };
        this.inputConsumer = input_ui_ipc_provider_1.createConsumer({ log: options.log, port: 3001 });
        this.inputConsumer.askSupported();
    }
    description(arg) {
        return this.name;
    }
    predicate(arg) {
        this.inputConsumer.askSupported();
        this.targetNameAndReorder = this.getTargetNameAndReorder(arg.sourceFile.fileName, typescript_ast_util_1.positionOrRangeToNumber(arg.positionOrRange));
        if (!this.targetNameAndReorder) {
            arg.log('predicate false because getTargetNameAndReorder did not found anything');
            return false;
        }
        return true;
    }
    getCompletionsAtPosition(fileName, position, options) {
        return this.getTextUITool().getCompletionsAtPosition(fileName, position, options);
    }
    getApplicableRefactors(info, refactorName, refactorActionName, fileName, positionOrRange, userPreferences) {
        this.inputConsumer.askSupported();
        const applicableRefactors = this.getTextUITool().getApplicableRefactors(info, refactorName, refactorActionName, fileName, positionOrRange, userPreferences);
        this.selectedAction = applicableRefactors.selectedAction;
        this.options.log('if there is no textUITool special comment it could be still a third party in' + !applicableRefactors.selectedAction + ' ' + this.targetNameAndReorder + ' ' + this.inputConsumer.hasSupport(input_ui_ipc_provider_1.INPUT_ACTIONS.inputText));
        // if there is no textUITool special comment it could be still a third party input provider ?
        if (!applicableRefactors.selectedAction && this.targetNameAndReorder && this.inputConsumer.hasSupport(input_ui_ipc_provider_1.INPUT_ACTIONS.inputText)
        // && !applicableRefactors.refactors.find(r=>r.name===refactorName && !!r.actions.find(a=>a.name===refactorActionName))
        ) {
            // this.options.log('if there is no textUITool special comment it could be still a third party in')
            // if(!applicableRefactors.refactors.find(r=>r.name===refactorName && !!r.actions.find(a=>a.name===refactorActionName))) 
            applicableRefactors.refactors.push({
                name: refactorName,
                description: `${refactorName} description`,
                actions: [{ name: refactorActionName, description: this.printRefactorSuggestionMessage(this.targetNameAndReorder.name) }]
            });
        }
        return applicableRefactors;
    }
    getTextUITool() {
        if (!this.interactionTool) {
            this.interactionTool = typescript_plugins_text_based_user_interaction_1.create(this.getTextUIToolConfig());
        }
        return this.interactionTool;
    }
    getTargetNode(fileName, position) {
        const sourceFile = this.options.info.languageService.getProgram().getSourceFile(fileName);
        const target = typescript_ast_util_1.findChildContainingRangeLight(sourceFile, typescript_ast_util_1.positionOrRangeToRange(position));
        const predicate = p => ts.isCallSignatureDeclaration(p) || ts.isFunctionLike(p) || ts.isCallOrNewExpression(p) || ts.isMethodSignature(p) || ts.isConstructSignatureDeclaration(p);
        // this.options.log('getTargetNode  concat(getAscendants, ' + [target].concat(getAscendants(target)).map(t => getKindName(t) + ' - ' + predicate(t)).join(', '))
        return typescript_ast_util_1.findAscendant(target, predicate, true);
    }
    getTargetNameAndReorder(fileName, position) {
        let targetNode = this.getTargetNode(fileName, position);
        if (!targetNode) {
            return;
        }
        let reorder = [];
        let name;
        if (ts.isFunctionLike(targetNode)) {
            if (!targetNode || targetNode.parameters && targetNode.parameters.length <= 1) {
                return;
            }
            name = targetNode.name ? targetNode.name.getText() : (targetNode.parent && targetNode.parent.name && targetNode.parent.name) ? targetNode.parent.name.getText() : undefined;
            if (!name) {
                return;
            }
            for (let i = 0; i < targetNode.parameters.length; i++) {
                reorder.push(targetNode.parameters.length - i - 1);
            }
        }
        else if (ts.isCallExpression(targetNode)) {
            if (!targetNode || targetNode.arguments && targetNode.arguments.length <= 1) {
                return;
            }
            if (!ts.isIdentifier(targetNode.expression)) {
                return;
            }
            name = targetNode.expression.getText();
            for (let i = 0; i < targetNode.arguments.length; i++) {
                reorder.push(targetNode.arguments.length - i - 1);
            }
        }
        //TODO: isCallNew and others that comply with : ts.isCallSignatureDeclaration(p)||ts.isFunctionLike(p)||ts.isCallOrNewExpression(p
        else {
            this.options.log('snippet undefined because not isCallExpression and not isFunctionLike' + targetNode.getText() + ' - ' + typescript_ast_util_1.getKindName(targetNode));
            return;
        }
        return { name, reorder, targetNode };
    }
    printRefactorSuggestionMessage(name) {
        return `Reorder parameters of "${name}"`;
    }
    getTextUIToolConfig() {
        return {
            prefix: '&%&%',
            log: this.options.log,
            actions: [
                {
                    name: 'reorderParams',
                    args: ['name', 'reorder'],
                    commentType: 'block',
                    print: action => this.printRefactorSuggestionMessage(action.args.name),
                    snippet: (fileName, position) => {
                        this.targetNameAndReorder = this.getTargetNameAndReorder(fileName, position);
                        if (!this.targetNameAndReorder) {
                            return;
                        }
                        const { name, reorder } = this.targetNameAndReorder;
                        const help = this.config.helpComment ? `
    
            /* Help: The second argument is the new order of parameters. Number N in index M means move the M-th 
                argument to index N. Examples: 
             * [1] means switch between first and second
             * [3, 2] means move the first parameter to fourth position and move the second parameter to the third. 
             *   (the rest of the parameters, (third and fourth) will move left to accommodate this requirements) */` : '';
                        return `reorderParams("${name}", [${reorder.join(', ')}])${help}`;
                    },
                    // TODO: we could give a more intuitive text-based API by letting the user provide the new signature. Then we create a new function with that signature in order to parse it correctly. 
                    nameExtra: (fileName, position) => {
                        if (!this.targetNameAndReorder) {
                            return '';
                        }
                        const { name } = this.targetNameAndReorder;
                        return `of ${name}`;
                        // const func = this.getTargetNode(fileName, position)
                        // if (func && ts.isFunctionLike(func)) {
                        //   const name = func.name ? func.name.getText() : (func.parent && (func.parent as any).name && (func.parent as any).name) ? (func.parent as any).name.getText() : ''
                        //   return `of ${name}`
                        // }
                        // else if (ts.isCallExpression(func)) {
                        //   return `of ${func.expression ? func.expression.getText() : ''}`
                        // }
                        // else {
                        //   return ''
                        // }
                    }
                }
            ]
        };
    }
    applyImpl(arg, reorder) {
        const sourceFile = arg.simpleNode.getSourceFile();
        const funcDecl = this.getSimpleTargetNode(sourceFile, typescript_ast_util_1.positionOrRangeToNumber(arg.positionOrRange), this.targetNameAndReorder.name, this.options.log);
        if (!funcDecl) {
            this.options.log(`reorderParamsPlugin applyImpl aborted because function ${this.targetNameAndReorder.name} cannot be found at ${arg.positionOrRange}`);
            return;
        }
        this.options.log(`reorderParamsPlugin applyImpl calling reorderParameters with reorder: [${reorder.join(', ')}]`);
        reorderParams_1.reorderParameters(funcDecl, reorder, this.options.log);
        sourceFile.saveSync();
    }
    apply(arg) {
        if (!this.selectedAction && this.inputConsumer.hasSupport(input_ui_ipc_provider_1.INPUT_ACTIONS.inputText)) {
            // this.options.log('this.inputConsumer.inputText before')
            this.inputConsumer.inputText({ prompt: 'Enter reorderParam definition', placeHolder: '[1]' }
            // , response=>{
            //   this.options.log('this.inputConsumer.inputText cb !!! ')
            //   // this.applyImpl(arg, JSON.parse(response.answer))
            // }
            )
                .then(response => {
                // this.options.log('this.inputConsumer.inputText then !!! 2 '+response)
                // this.options.log('this.inputConsumer.inputText then !!! ' +((response && response.answer) ? response.answer : 'undefinennen' ))
                this.applyImpl(arg, JSON.parse(response.answer));
            }).catch(ex => {
                this.options.log('this.inputConsumer.inputText catch ' + ex);
            });
            // this.options.log('this.inputConsumer.inputText affter')
        }
        else {
            this.applyImpl(arg, this.selectedAction.args.reorder);
        }
    }
    getSimpleTargetNode(file, position, name, log) {
        let expr = file.getDescendantAtPos(position);
        if (!expr) {
            return;
        }
        const predicate = e => (ts_simple_ast_1.TypeGuards.isSignaturedDeclaration(e) || ts_simple_ast_1.TypeGuards.isPropertySignature(e) || ts_simple_ast_1.TypeGuards.isFunctionLikeDeclaration(e)) && (ts_simple_ast_1.TypeGuards.isNamedNode(e) || ts_simple_ast_1.TypeGuards.isNameableNode(e) || ts_simple_ast_1.TypeGuards.isPropertyNamedNode(e)) && (!name || e.getName() === name);
        const e = [expr].concat(expr.getAncestors()).find(predicate);
        if (!e) {
            return;
        }
        return e;
    }
}
exports.ReorderParamsCodeFixImpl = ReorderParamsCodeFixImpl;
//# sourceMappingURL=reorderParamsPlugin.js.map